#!/usr/bin/env python
# -*- coding: utf-8 -*-

import subprocess
import os
import time

ldap_domain = None
ldap_dn = None
ldap_pw = None
ldap_base = None


def process_lines():
    global ldap_domain, ldap_dn, ldap_pw, ldap_base
    with open("/etc/nslcd.conf", 'r') as file:
        lines = file.readlines()
        for line in lines:
            my_list = line.strip().split()
            if "uri" in my_list:
                ldap_domain = my_list[1]
            if "binddn" in my_list:
                ldap_dn = my_list[1]
            if "bindpw" in my_list:
                ldap_pw = my_list[1]
            if "base" in my_list:
                ldap_base = my_list[1]


def get_ldap_information():
    # 判断是否存在"/etc/nslcd.conf"文件
    if os.path.exists("/etc/nslcd.conf"):
        file = open(os.devnull, 'w')
        # 判断文件内容是否存在
        ret1 = subprocess.Popen(["cat", "/etc/nslcd.conf"], shell=False, stdout=subprocess.PIPE)
        ret2 = subprocess.Popen(["cat", "/etc/nslcd.conf"], shell=False, stdout=subprocess.PIPE)
        ret3 = subprocess.Popen(["cat", "/etc/nslcd.conf"], shell=False, stdout=subprocess.PIPE)
        ret4 = subprocess.Popen(["cat", "/etc/nslcd.conf"], shell=False, stdout=subprocess.PIPE)
        exist_check = {
            "uri": subprocess.call("grep uri".split(), shell=False, stdin=ret1.stdout, stdout=file, stderr=file) == 0,
            "binddn": subprocess.call("grep binddn".split(), shell=False, stdin=ret2.stdout, stdout=file, stderr=file) == 0,
            "bindpw": subprocess.call("grep bindpw".split(), shell=False, stdin=ret3.stdout, stdout=file, stderr=file) == 0,
            "base": subprocess.call("grep base".split(), shell=False, stdin=ret4.stdout, stdout=file, stderr=file) == 0
        }
        file.close()
        if all(exist_check.values()):
            # 从"/etc/nslcd.conf"文件中获取ldap相关信息,包括域名、DN、ldap管理员用户、ldap密码等
            process_lines()
        else:
            print("\033[91m[unhealthy]\033[0m[service check][ldap client][nslcd.conf]: The file content is missing.")
            exit(255)
    else:
        print("\033[91m[unhealthy]\033[0m[service check][ldap client][nslcd.conf]: The file does not exist.")
        exit(255)


def check_ldap_client():
    ldap_client_service = ["nslcd.service", "oddjobd.service"]
    for client_service in ldap_client_service:
        file = open(os.devnull, 'w')
        status_service = subprocess.call("systemctl status {}".format(client_service).split(), shell=False,
                                         stdout=file, stderr=file)
        is_enable_service = subprocess.call("systemctl is-enabled {}".format(client_service).split(), shell=False,
                                            stdout=file, stderr=file)
        file.close()
        if status_service == 0:
            if is_enable_service == 0:
                print("\033[32m[healthy]\033[0m[service check][ldap client][{}]: {} is running and enabled.".format(
                    client_service, client_service))
            else:
                print("\033[91m[unhealthy]\033[0m[service check][ldap client][{}]: {} is not enabled.".format(
                    client_service, client_service))
        elif status_service == 3:
            if is_enable_service == 0:
                print(
                    "\033[91m[unhealthy]\033[0m[service check][ldap client][{}]: {} not running.".format(client_service,
                                                                                                         client_service))
            else:
                print(
                    "\033[91m[unhealthy]\033[0m[service check][ldap client][{}]: {} not running and not enabled".format(
                        client_service, client_service))
        else:
            print("\033[91m[unhealthy]\033[0m[service check][ldap client][{}]: could not be found.".format(
                client_service))
            continue

    # 连接ldap服务端验证
    get_ldap_information()
    timeout = 15  # 设置超时时间为15秒
    file = open(os.devnull, 'w')
    # ldapsearch在列表中不被识别
    ldap_connect = subprocess.Popen(["ldapsearch", "-b", ldap_base, "-D", ldap_dn, "-H", ldap_domain, "-w",
                                     ldap_pw], shell=False, stdout=file, stderr=file)
    file.close()
    start_time = time.time()
    while True:
        elapsed_time = time.time() - start_time
        if elapsed_time > timeout:
            print(
                "\033[91m[unhealthy]\033[0m[service check][ldap client][ldap connect]: connect ldap server failed within 15s.")
            exit(255)
        if ldap_connect.poll() == 0:
            print(
                "\033[32m[healthy]\033[0m[service check][ldap client][ldap connect]: connect ldap server successfully.")
            exit(255)


if __name__ == '__main__':
    check_ldap_client()
