#!/usr/bin/env python
# -*- coding: utf-8 -*-

import subprocess

healthy_pre = '\033[32m[healthy]\033[0m[service check][time sync]:'
unhealthy_pre = '\033[91m[unhealthy]\033[0m[service check][time sync]:'


def check_timezone():
    timezone = subprocess.check_output(["timedatectl", "status"], shell=False).decode("utf-8")
    if "CST" in timezone:
        print('\033[32m[healthy]\033[0m[service check][timezone]: timezone is CST.')
    else:
        print('\033[91m[unhealthy]\033[0m[service check][timezone]: timezone is not CST.')


def check_service_status():
    ntp_status_process = subprocess.Popen(["systemctl", "status", "ntpd.service"], shell=False, stdout=subprocess.PIPE)
    ntp_status = ntp_status_process.communicate(timeout=5)[0].decode('utf-8')

    chrony_status_process = subprocess.Popen(["systemctl", "status", "chronyd.service"], shell=False
                                             , stdout=subprocess.PIPE)
    chrony_status = chrony_status_process.communicate(timeout=5)[0].decode('utf-8')

    if "running" in ntp_status and "running" in chrony_status:
        print(unhealthy_pre + 'Both service NTP and service Chrony exist.')

    elif "running" in ntp_status:
        print(healthy_pre + 'NTP Service is running.')
        check_start("ntpd.service")
        check_ntp_sync()

    elif "running" in chrony_status:
        print(healthy_pre + 'Chrony Service is running.')
        check_start("chronyd.service")
        check_chrony_sync()

    else:
        print(unhealthy_pre + 'No time synchronization service is detected.')


def check_start(service):
    status = subprocess.run(["systemctl", "is-enabled", service], shell=False, capture_output=True, text=True)
    if "enabled" in status.stdout:
        print(healthy_pre + service + 'is Automatic Startup.')
    else:
        print(unhealthy_pre + service + 'is Not Automatic Startup.')


def check_ntp_sync():
    result = subprocess.run(["ntpq", "-p"], shell=False, capture_output=True, text=True)
    lines = result.stdout.splitlines()
    if lines[2][0] == "*":
        print(healthy_pre + "NTP Client is synchronized.")
    else:
        print(unhealthy_pre + "NTP Client is not synchronized.")


def check_chrony_sync():
    result = subprocess.run(["chronyc", "sources"], shell=False, capture_output=True, text=True)
    lines = result.stdout.splitlines()
    if len(lines) < 3:
        print(unhealthy_pre + "chronyc sources No content is displayed.")
        return

    p = lines[2].split()[0]
    if len(lines) < 4:
        n = ""
    else:
        n = lines[3].split()[0]

    if "^*" in p or "^*" in n:
        print(healthy_pre + "Chrony Client is synchronized.")
    else:
        print(unhealthy_pre + "Chrony Client is not synchronized.")


if __name__ == "__main__":
    check_timezone()
    check_service_status()
