#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Obtain the Chrony service Configuration of the Server Node.
"""
import os
import subprocess

return_message = {
    'chrony dependency package': 'UNINSTALLED',
    '/etc/chrony.conf': 'NULL',
    '/etc/chrony.conf master ip': 'NULL',
    '/etc/chrony.conf slave ip': 'NULL',
    'time zone': 'NULL'
}


def master_config_item():
    """
    获取三方件服务Chrony主节点配置项
    :return:
    """
    process1 = subprocess.Popen(['/usr/bin/cat', '/etc/chrony.conf'], shell=False, stdout=subprocess.PIPE)
    process = subprocess.Popen(['/usr/bin/grep', '^server'], shell=False, stdin=process1.stdout, stdout=subprocess.PIPE)
    try:
        status = subprocess.check_output(['/usr/bin/grep', 'prefer$'], stdin=process.stdout).decode("utf-8").strip()
        return_message['/etc/chrony.conf master ip'] = status
    except subprocess.CalledProcessError:
        return_message['/etc/chrony.conf master ip'] = 'NULL'
    process.stdout.close()


def slave_config_item():
    """
    获取三方件服务Chrony备节点配置项
    :return:
    """
    process1 = subprocess.Popen(['/usr/bin/cat', '/etc/chrony.conf'], shell=False, stdout=subprocess.PIPE)
    process = subprocess.Popen(['/usr/bin/grep', '^server'], shell=False, stdin=process1.stdout, stdout=subprocess.PIPE)
    try:
        status = subprocess.check_output(['/usr/bin/grep', 'iburst$'], stdin=process.stdout).decode("utf-8").strip()
        return_message['/etc/chrony.conf slave ip'] = status
    except subprocess.CalledProcessError:
        return_message['/etc/chrony.conf slave ip'] = 'NULL'
    process.stdout.close()


def get_chrony_items():
    """
    获取服务器节点三方件服务Chrony配置项
    :return: 返回组织后的服务器节点三方件服务Chrony配置项
    """
    # 获取系统时区信息
    status = subprocess.check_output(['/usr/bin/date', '+%Z'], shell=False).decode("utf-8").strip()
    return_message['time zone'] = status

    devnull = os.open(os.devnull, os.O_WRONLY, os.O_CREAT)
    try:
        subprocess.check_output(['/usr/bin/yum', 'list', 'installed', 'chrony'], shell=False, stderr=devnull)
        return_message['chrony dependency package'] = 'INSTALLED'
        os.close(devnull)
    except subprocess.CalledProcessError:
        os.close(devnull)
        return

    chrony_file = '/etc/chrony.conf'
    if not os.path.exists(chrony_file):
        return

    return_message[chrony_file] = 'EXIST'
    master_config_item()
    slave_config_item()


get_chrony_items()
print(return_message)