#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Obtain the openLDAP service client configuration of the server node.
"""
import grp
import hashlib
import os
import subprocess
import pwd

return_message = {
    'firewalld service disabled': 'True',
    'firewalld service auto-start disabled': 'True',
    'selinux service disabled': 'True',
    '/etc/openldap/ldap.conf': 'NULL',
    '/etc/openldap/ldap.conf chmod': 'NULL',
    '/etc/nslcd.conf': 'NULL',
    '/etc/nslcd.conf chmod': 'NULL',
    '/etc/openldap/cacerts/ca.crt': 'NULL',
    '/etc/openldap/cacerts/ca.crt chmod': 'NULL',
}


def content_encode(content):
    """
    对传入的字符串内容进行编码
    :param content: 代编码的字符串内容
    :return: 返回编号后的内容
    """
    encode = hashlib.md5()
    encode.update(content.encode('utf-8'))
    return encode.hexdigest()


def get_chmod(passfree_file):
    """
    获取文件或文件夹权限
    :param passfree_file: 待获取的文件或文件夹
    :return:
    """
    stat_info = os.stat(passfree_file)
    mode = stat_info.st_mode
    # 将权限转换为八进制数
    octal_mode = oct(mode)[-3:]
    group_name = grp.getgrgid(stat_info.st_gid).gr_name
    user_name = pwd.getpwuid(stat_info.st_uid).pw_name
    return_message['{} chmod'.format(passfree_file)] = '{} {} {}'.format(octal_mode, user_name, group_name)


def get_file_items(ldap_file):
    """
    获取三方件服务openLDAP服务客户端相关文件及文件权限
    :param ldap_file: 三方件服务openLDAP服务客户端相关配置文件
    :return:
    """
    if not os.path.isfile(ldap_file):
        return
    with open(ldap_file) as file:
        return_message['{}'.format(ldap_file)] = content_encode(file.read())
    get_chmod(ldap_file)


def service_check(service_name):
    """
    检查相关服务是否关闭及开机自启项配置
    :return:
    """
    devnull = os.open(os.devnull, os.O_WRONLY, os.O_CREAT)
    return_code = subprocess.call(['/usr/bin/systemctl', 'is-active', service_name], shell=False, stdout=devnull)
    if return_code == 0:
        return_message['firewalld service disabled'] = 'False'

    return_code = subprocess.call(['/usr/bin/systemctl', 'is-enabled', service_name], shell=False, stdout=devnull)
    if return_code == 0:
        return_message['firewalld service auto-start disabled'] = 'False'
    os.close(devnull)


def ldap_client_configuration_item():
    """
    获取服务器节点三方件服务openLDAP服务客户端相关配置项
    :return: 返回组装后的服务器节点三方件服务openLDAP服务客户端配置项
    """
    # 防火墙关闭检查
    service_check('firewalld.service')
    # selinux配置检查
    output = subprocess.check_output(['/usr/sbin/getenforce'], shell=False).decode("utf-8").strip()
    if 'Enforcing' in output:
        return_message['selinux service disabled'] = 'False'

    get_file_items('/etc/openldap/ldap.conf')
    get_file_items('/etc/nslcd.conf')
    get_file_items('/etc/openldap/cacerts/ca.crt')


ldap_client_configuration_item()
print(return_message)
