#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import sys
import subprocess

# yum_server_ip （必填项）YUM源服务端IP地址
REMOTE_PARAMETERS = " ".join(sys.argv[1:])

ARGV_EXAMPLE_DIC = {"yum_server_ip": "xxxx"}
# 存放输入的参数值
ARGV_INPUT_DIC = ARGV_EXAMPLE_DIC


def script_params_example():
    message = "parameter example: "
    for key, value in ARGV_EXAMPLE_DIC.items():
        message = message + key + "=" + value + " "
    return message.strip()


def validate_script_params():
    if len(REMOTE_PARAMETERS) == 0:
        print('Please enter parameters, ' + script_params_example())
        exit(255)
    tmp_argv = REMOTE_PARAMETERS.split(" ")
    for item in tmp_argv:
        argv = item.split("=")
        if len(argv) != 2:
            print('Invalid parameters format, ' + script_params_example())
            exit(255)
        if argv[0] not in ARGV_EXAMPLE_DIC.keys():
            print('Parameter [ ' + argv[0] + ' ] is not defined, ' + script_params_example())
            exit(255)
        if argv[1].strip() == "" and ARGV_EXAMPLE_DIC.get(argv[0]) == "xxxx":
            print('Parameter [ ' + argv[0] + ' ] cannot be empty.')
            exit(255)
        ARGV_INPUT_DIC[argv[0]] = argv[1]

    for key, value in ARGV_INPUT_DIC.items():
        if value == "xxxx":
            print('Parameter [ ' + key + ' ] cannot be empty.')
            exit(255)

    if len(ARGV_EXAMPLE_DIC) != len(ARGV_INPUT_DIC):
        print('Number of parameters does not match, ' + script_params_example())
        exit(255)
    validate_business_params()


def validate_business_params():
    ip_addr = ARGV_INPUT_DIC.get("yum_server_ip")
    parts = ip_addr.split('.')
    if len(parts) != 4:
        print('Incorrect IP address format, ip = {}'.format(ip_addr))
        exit(255)
    for part in parts:
        if not part.isdigit():
            print('Incorrect IP address format, ip = {}'.format(ip_addr))
            exit(255)
        if int(part) < 0 or int(part) > 255:
            print('Incorrect IP address format, ip = {}'.format(ip_addr))
            exit(255)


validate_script_params()
# 服务器IP地址
yum_server_ip = ARGV_INPUT_DIC.get("yum_server_ip")
# 服务器http中镜像地址

os_release = subprocess.check_output("cat /etc/os-release".split(), shell=False).decode("utf-8").strip()
if "openEuler" in os_release:
    yum_server_location = "openEuler"
elif "Kylin" in os_release:
    yum_server_location = "Kylin"
elif "CentOS" in os_release:
    yum_server_location = "CentOS"
else:
    yum_server_location = "OS"

# 网络YUM源访问地址
yum_http_url = "http://{}/{}/".format(yum_server_ip, yum_server_location)

# 备份YUM配置文件
if not os.path.isdir('/etc/yum.repos.bak/'):
    os.makedirs('/etc/yum.repos.bak/', mode=0o755)

if len(os.listdir("/etc/yum.repos.d/")) != 0:
    with open(os.devnull, 'w') as file:
        ret1 = subprocess.Popen(['ls', '/etc/yum.repos.d/'], stdout=subprocess.PIPE).stdout.read().decode().split('\n')
        ret1 = [file for file in ret1 if file]
        for ret in ret1:
            subprocess.call(['mv', '-f', '/etc/yum.repos.d/{}'.format(ret), '/etc/yum.repos.bak/'], shell=False)

# 配置YUM配置文件
ret1 = subprocess.Popen("cat /etc/system-release".split(), shell=False, stdout=subprocess.PIPE)
ret = subprocess.call("grep 'CentOS Linux release 8.2.2004'".split(' ', 1), shell=False, stdin=ret1.stdout)
if ret == 0:
    ##### 支持CentOS_V8.2 ####
    with open("/etc/yum.repos.d/http.repo", 'w') as file:
        file.write("[http-BaseOS]\nname=centos-BaseOS\nbaseurl={}\nenabled=1\ngpgcheck=0\n\n".format(
        yum_http_url))
    with open("/etc/yum.repos.d/http.repo", 'a') as file:
        file.write("[http-AppStream]\nname=centos-AppStream\nbaseurl={}\nenabled=1\ngpgcheck=0\n\n".format(
        yum_http_url))
else:
    with open("/etc/yum.repos.d/http.repo", 'w') as file:
        file.write("[http]\nname=http\nbaseurl={}\nenabled=1\ngpgcheck=0\n\n".format(
        yum_http_url))

# 清理缓存重新加载
subprocess.call("yum clean all".split(), shell=False)
subprocess.call("yum makecache".split(), shell=False)
print("Configuration Completed!")
