#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import re
import sys
import shutil
import subprocess

# yum_iso_path （必填项）YUM源ISO文件所在路径（包含ISO文件名的全路径）
REMOTE_PARAMETERS = " ".join(sys.argv[1:])

ARGV_EXAMPLE_DIC = {"yum_iso_path": "xxxx"}
# 存放输入的参数值
ARGV_INPUT_DIC = ARGV_EXAMPLE_DIC


def script_params_example():
    message = "parameter example: "
    for key, value in ARGV_EXAMPLE_DIC.items():
        message = message + key + "=" + value + " "
    return message.strip()


def validate_script_params():
    if len(REMOTE_PARAMETERS) == 0:
        print('Please enter parameters, ' + script_params_example())
        exit(255)
    tmp_argv = REMOTE_PARAMETERS.split(" ")
    for item in tmp_argv:
        argv = item.split("=")
        if len(argv) != 2:
            print('Invalid parameters format, ' + script_params_example())
            exit(255)
        if argv[0] not in ARGV_EXAMPLE_DIC.keys():
            print('Parameter [ ' + argv[0] + ' ] is not defined, ' + script_params_example())
            exit(255)
        if argv[1].strip() == "" and ARGV_EXAMPLE_DIC.get(argv[0]) == "xxxx":
            print('Parameter [ ' + argv[0] + ' ] cannot be empty.')
            exit(255)
        ARGV_INPUT_DIC[argv[0]] = argv[1]

    for key, value in ARGV_INPUT_DIC.items():
        if value == "xxxx":
            print('Parameter [ ' + key + ' ] cannot be empty.')
            exit(255)

    if len(ARGV_EXAMPLE_DIC) != len(ARGV_INPUT_DIC):
        print('Number of parameters does not match, ' + script_params_example())
        exit(255)
    validate_business_params()


def validate_business_params():
    iso_file = ARGV_INPUT_DIC.get("yum_iso_path")
    if not os.path.exists(iso_file):
        print('[{}] file does not exist.'.format(iso_file))
        exit(255)


validate_script_params()
# YUM源镜像完整路径
yum_iso_full_path = ARGV_INPUT_DIC.get("yum_iso_path")
# YUM镜像文件名称
yum_iso_file_name = os.path.basename(yum_iso_full_path)

# http服务存放位置
if yum_iso_file_name.startswith("openEuler"):
    yum_server_location = "openEuler"
elif yum_iso_file_name.startswith("Cent"):
    yum_server_location = "CentOS"
elif yum_iso_file_name.startswith("Kylin"):
    yum_server_location = "Kylin"
else:
    yum_server_location = "OS"

# YUM镜像服务器安装挂载路径
yum_mount_path = "/mnt"
# 服务器网络源路径
yum_http_path = "/var/www/html/{}".format(yum_server_location)


# 配置开机自启动函数
def config_auto_startup():
    # 加固判断文件是否存在
    if not os.path.exists("/etc/rc.d/rc.local"):
        print('The source file or folder does not exist.')
        exit(255)
    with open('/etc/rc.d/rc.local', 'r') as f:
        lines = f.readlines()
    line_nums = [i + 1 for i, line in enumerate(lines) if re.search('mount -t iso9660 -o loop', line)]
    if line_nums:
        for num in reversed(line_nums):
            # 删除之前配置的自启动配置内容
            lines.pop(num - 1)
        # 删除多余空白行
        filtered_lines = []
        for line in lines:
            if line.strip():
                filtered_lines.append(line)
        with open('/etc/rc.d/rc.local', 'w') as f:
            f.write(''.join(filtered_lines))
    if 'mount -t iso9660 -o loop {} {}'.format(yum_iso_full_path, yum_mount_path) in lines[-1]:
        print('Automatic mounting upon startup has been configured.')
    else:
        # 追加内容到配置文件中
        with open('/etc/rc.d/rc.local', 'a') as f:
            f.write('mount -t iso9660 -o loop {} {}'.format(yum_iso_full_path, yum_mount_path))
            subprocess.call('chmod +x /etc/rc.d/rc.local'.split(), shell=False)
            


################## 配置中心源本地YUM ####################
# 如果已挂载则先卸载,目的是防止与自动化初始化脚本挂载方式不一致导致不可预知的问题
try:
    ret1 = subprocess.Popen(["df", "-h"], shell=False, stdout=subprocess.PIPE)
    mount_result = subprocess.check_output(["grep", "-o", "{}".format(yum_mount_path)], shell=False, stdin=ret1.stdout)
    if mount_result != "":
        with open(os.devnull, 'w') as file:
            subprocess.call("umount -l {}".format(yum_mount_path).split(), stderr=file, shell=False)
except subprocess.CalledProcessError:
    pass

# 挂载YUM镜像
subprocess.call("mount -t iso9660 -o loop {} {}".format(yum_iso_full_path, yum_mount_path).split(), shell=False)

# 检查是否挂载成功
try:
    ret1 = subprocess.Popen("df -h".split(), stdout=subprocess.PIPE, shell=False)
    subprocess.check_output('grep -o {}'.format(yum_mount_path).split(), stdin=ret1.stdout, shell=False)
except subprocess.CalledProcessError:
    print('Yum mounted failed, check the mounting process.')
    exit(255)

if not os.path.isdir('/etc/yum.repos.bak/'):
    os.makedirs('/etc/yum.repos.bak/', mode=0o755)

# 备份旧的repo配置文件
with open(os.devnull, 'w') as file:
    ret1 = subprocess.Popen('ls /etc/yum.repos.d/'.split(), stdout=subprocess.PIPE).stdout.read().decode().split('\n')
    ret1 = [file for file in ret1 if file]
    for ret in ret1:
        subprocess.call(['mv', '-f', '/etc/yum.repos.d/{}'.format(ret), '/etc/yum.repos.bak/'], shell=False, stderr=file)

# 生成新的repo配置文件
with open('/etc/system-release') as file:
    if 'CentOS Linux release 8.2.2004' in file.read():
        # 支持CentOS_V8.2
        with open("/etc/yum.repos.d/local.repo", 'w') as file:
            file.write("[centos-BaseOS]\nname=centos-BaseOS\nbaseurl=file://{}/BaseOS/\nenabled=1\ngpgcheck=0\n\n".format(
                yum_mount_path))
        with open("/etc/yum.repos.d/local.repo", 'a') as file:
            file.write("[centos-AppStream]\nname=centos-AppStream\nbaseurl=file://{}/AppStream/\nenabled=1\ngpgcheck=0\n\n".format(
                yum_mount_path))
    else:
        with open("/etc/yum.repos.d/local.repo", 'w') as file:
            file.write("[local]\nname=local\nbaseurl=file://{}\nenabled=1\ngpgcheck=0\n\n".format(
                yum_mount_path))

# 清理和重新加载缓存
subprocess.call('yum clean all'.split(), shell=False)
subprocess.call("yum makecache".split(), shell=False)

# 配置开机自动启动
config_auto_startup()

################# 配置中心源HTTP服务 ######################
# 检查安装httpd服务

httpd_installed = subprocess.check_output('rpm -qa httpd'.split(), shell=False, universal_newlines=True)
if not httpd_installed:
    subprocess.check_output(['yum', 'install', '-y', 'httpd'], shell=False)
    subprocess.check_output(['systemctl', 'enable', 'httpd.service'], shell=False)

# 启动http服务
subprocess.call("systemctl stop firewalld".split(), shell=False)
subprocess.call("systemctl start httpd.service".split(), shell=False)

# 将镜像文件复制到http目录
if os.path.isdir(yum_http_path):
    subprocess.call("rm -rf {}".format(yum_http_path).split(), shell=False)

subprocess.call("mkdir -p {}".format(yum_http_path).split(), shell=False)
print("loading...")
shutil.copytree(yum_mount_path, yum_http_path, dirs_exist_ok=True)
print("Configuration Completed!")
