#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import subprocess
import sys
import tempfile

# cuda_package_path （必填项）CUDA Toolkit安装包所在路径
REMOTE_PARAMETERS = " ".join(sys.argv[1:])

ARGV_EXAMPLE_DIC = {"cuda_package_path": "xxxx"}
# 存放输入的参数值
ARGV_INPUT_DIC = ARGV_EXAMPLE_DIC


def script_params_example():
    message = "parameter example: "
    for key, value in ARGV_EXAMPLE_DIC.items():
        message = message + key + "=" + value + " "
    return message.strip()


def validate_script_params():
    if len(REMOTE_PARAMETERS) == 0:
        print('Please enter parameters, ' + script_params_example())
        exit(255)
    tmp_argv = REMOTE_PARAMETERS.split(" ")
    for item in tmp_argv:
        argv = item.split("=")
        if len(argv) != 2:
            print('Invalid parameters format, ' + script_params_example())
            exit(255)
        if argv[0] not in ARGV_EXAMPLE_DIC.keys():
            print('Parameter [ ' + argv[0] + ' ] is not defined, ' + script_params_example())
            exit(255)
        if argv[1].strip() == "" and ARGV_EXAMPLE_DIC.get(argv[0]) == "xxxx":
            print('Parameter [ ' + argv[0] + ' ] cannot be empty.')
            exit(255)
        ARGV_INPUT_DIC[argv[0]] = argv[1]

    for key, value in ARGV_INPUT_DIC.items():
        if value == "xxxx":
            print('Parameter [ ' + key + ' ] cannot be empty.')
            exit(255)

    if len(ARGV_EXAMPLE_DIC) != len(ARGV_INPUT_DIC):
        print('Number of parameters does not match, ' + script_params_example())
        exit(255)
    validate_business_params()


def validate_business_params():
    if not has_gpu():
        print("No GPU NIC is installed on the current node.")
        exit(255)
    cuda_driver = ARGV_INPUT_DIC.get("cuda_package_path")
    if not os.path.exists(cuda_driver):
        print('CUDA Toolkit driver installation package does not exist.')
        exit(255)
    if os.path.isdir(cuda_driver):
        print('Please specify the path to the CUDA Toolkit driver installation package, not the directory.')
        exit(255)


# 检查环境是否存在GPU卡
def has_gpu():
    cmd = "lspci | grep -i nvidia"
    try:
        ret1 = subprocess.Popen(["lspci"], shell=False, stdout=subprocess.PIPE)
        subprocess.check_output(["grep", "-i", "nvidia"], shell=False, stdin=ret1.stdout)
        return True
    except subprocess.CalledProcessError:
        return False


# 安装依赖包
def install_cuda_dependency_libs():
    # 先判断环境是否有依赖和yum源是否可用
    file = open(os.devnull, 'w')
    ret = subprocess.Popen('rpm -qa'.split(), shell=False, stdout=subprocess.PIPE)
    exist_gcc = subprocess.call('grep gcc-c++'.split(), shell=False, stdin=ret.stdout, stdout=file, stderr=file)
    status_yum = subprocess.call('yum makecache'.split(), shell=False, stdout=file, stderr=file)
    if exist_gcc != 0:
        if status_yum != 0:
            print("yum source is not available, couldn't install cuda_toolkit driver.")
            exit(255)
        if status_yum == 0:
            subprocess.call("yum install -y gcc-c++ kernel-source kernel-devel".split(), shell=False, stdout=file, stderr=file)
    file.close()


# 校验是否有Nouveau驱动，有则禁用
def install_cuda(cuda_driver_name):
    print("Installing CUDA_Toolkit driver, it may take several minutes, please wait....")
    # 为避免安装NVIDIA Driver时发生冲突，禁用Nouveau驱动
    file = open(os.devnull, 'w')
    ret = subprocess.Popen(["lsmod"], shell=False, stdout=subprocess.PIPE)
    if subprocess.call(["grep", "nouveau"], shell=False, stdin=ret.stdout) == 0:
        if os.path.exists("/etc/modprobe.d/blacklist.conf"):
            with open("/etc/modprobe.d/blacklist.conf", "r") as text:
                content = text.read()
            if "blacklist nouveau  options nouveau modeset=0" not in content:
                # 追加配置信息到文件中
                file1 = open("/etc/modprobe.d/blacklist.conf", 'a')
                file1.write("blacklist nouveau  options nouveau modeset=0")
                file1.close()
        # 备份刷新initramfs文件
        subprocess.call("mv /boot/initramfs-$(uname -r).img /boot/initramfs-$(uname -r).img.bak &>/dev/null",
                        shell=False, stdout=file, stderr=file)
        subprocess.call(["dracut", "/boot/initramfs-$(uname -r).img", "$(uname -r)"], shell=False,
                        stdout=file, stderr=file)

    # 校验当前节点是否安装CUDA_Toolkit
    status_nvcc = subprocess.call(["{}/bin/nvcc".format(cuda_install_path), "-V"], shell=False,
                                  stdout=file, stderr=file)
    process = subprocess.Popen(["{}/samples/1_Utilities/deviceQueryDrv/deviceQueryDrv".format(cuda_install_path)],
        shell=False, stdout=subprocess.PIPE)
    status_samples = subprocess.call(["grep", "PASS"], shell=False, stdin=process.stdout, stdout=file, stderr=file)
 
    if status_nvcc != 0 or status_samples != 0:
        # 在当前节点执行安装CUDA_Toolkit
        install_cuda_dependency_libs()
        subprocess.call(["chmod", "+x", os.path.join(tempfile.gettempdir(), cuda_driver_name)],
            shell=False, stdout=file, stderr=file)
        subprocess.call([os.path.join(tempfile.gettempdir(), cuda_driver_name), "--toolkit", "--samples", "--silent",
            "--override", "--driver", "--installpath={}".format(cuda_install_path)], shell=False)

    # 已安装或安装完成后，验证是否安装成功
    command1 = 'cd {}/samples/1_Utilities/deviceQueryDrv'.format(cuda_install_path)
    command2 = '{}/samples/1_Utilities/deviceQueryDrv/deviceQueryDrv'.format(cuda_install_path)
    if subprocess.call(command1.split(), shell=False, stdout=file, stdin=file) != 0:
        print('CUDA_Toolkit driver installation failed.')
    elif subprocess.call(command2.split(), shell=False, stdout=file, stdin=file) != 0:
        print('CUDA_Toolkit driver installation failed.')
    else:
        print('CUDA_Toolkit driver has been installed successfully.')
    file.close()


# 主函数入口
validate_script_params()
cuda_driver_full_path = ARGV_INPUT_DIC.get("cuda_package_path")
# CUDA_Toolkit驱动安装路径
cuda_install_path = "/usr/local/cuda"
subprocess.call('cp {} /tmp/'.format(cuda_driver_full_path).split(), shell=False)
install_cuda(os.path.basename(cuda_driver_full_path))
