#!/usr/bin/env python
# -*- coding: utf-8 -*-


import os
import re
import subprocess
import sys

REMOTE_PARAMETERS = " ".join(sys.argv[1:])

ARGV_EXAMPLE_DIC = {"internet_ip": "zzzz", "internet_domain_name": "zzzz"}
# 存放输入的参数值
ARGV_INPUT_DIC = ARGV_EXAMPLE_DIC


def script_params_example():
    message = "parameter example: "
    for key, value in ARGV_EXAMPLE_DIC.items():
        message = message + key + "=" + value + " "
    return message.strip()


def validate_script_params():
    if len(REMOTE_PARAMETERS) == 0:
        validate_business_params()

    else:
        tmp_argv = REMOTE_PARAMETERS.split(" ")
        for item in tmp_argv:
            argv = item.split("=")
            if len(argv) != 2:
                print('Invalid parameters format, ' + script_params_example())
                exit(255)
            if argv[0] not in ARGV_EXAMPLE_DIC.keys():
                print('Parameter [ ' + argv[0] + ' ] is not defined, ' + script_params_example())
                exit(255)
            if argv[1].strip() == "" and ARGV_EXAMPLE_DIC.get(argv[0]) == "xxxx":
                print('Parameter [ ' + argv[0] + ' ] cannot be empty.')
                exit(255)
            ARGV_INPUT_DIC[argv[0]] = argv[1]

        for key, value in ARGV_INPUT_DIC.items():
            if value == "xxxx":
                print('Parameter [ ' + key + ' ] cannot be empty.')
                exit(255)

        if len(ARGV_EXAMPLE_DIC) != len(ARGV_INPUT_DIC):
            print('Number of parameters does not match, ' + script_params_example())
            exit(255)
        validate_business_params()


def validate_business_params():
    if ARGV_INPUT_DIC.get("internet_domain_name") != 'zzzz':
        internet_domain_name = ARGV_INPUT_DIC.get("internet_domain_name")
        pattern = r'^(?:[a-z0-9](?:[a-z0-9-]*[a-z0-9])?\.)+[a-z0-9](?:[a-z0-9-]*[a-z0-9])?$'
        if not bool(re.match(pattern, internet_domain_name)):
            print("\033[1;31mError:\033[0m Invalid Domain Name")
            exit(255)

        print("Checking Domain Name connectivity... Please wait.")
        try:
            output = subprocess.check_output(["ping", "-c", "1", internet_domain_name], shell=False,
                stderr=subprocess.STDOUT, universal_newlines=True, timeout=2)
        except (subprocess.CalledProcessError, subprocess.TimeoutExpired):
            print("\033[1;31mError:\033[0m The Domain Name cannot be connected.")
            exit(255)

    if ARGV_INPUT_DIC.get("internet_ip") != 'zzzz':
        internet_ip = ARGV_INPUT_DIC.get("internet_ip")
        pattern = r'^(?:(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.){3}(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)$'
        if not bool(re.match(pattern, internet_ip)):
            print("\033[1;31mError:\033[0m Invalid IPv4 address")
            exit(255)

        print("Checking IP connectivity... Please wait.")
        try:
            output = subprocess.check_output(["ping", "-c", "1", internet_ip], shell=False,
                stderr=subprocess.STDOUT, universal_newlines=True, timeout=2)
        except (subprocess.CalledProcessError, subprocess.TimeoutExpired):
            print("\033[1;31mError:\033[0m The internet_ip cannot be connected.")
            exit(255)


def check_ntp():
    process1 = subprocess.Popen(["rpm", "-qa"], shell=False, stdout=subprocess.PIPE)
    process2 = subprocess.Popen(["grep", "ntp"], shell=False, stdin=process1.stdout, stdout=subprocess.PIPE)
    process2.communicate(timeout=5)
    returncode = process2.returncode

    if returncode == 0:
        # 关闭服务并禁止开机启动
        subprocess.call(["systemctl", "stop", "ntpd.service"], shell=False)
        subprocess.call(["systemctl", "disable", "ntpd.service"], shell=False)
        print("=============================== Stop && Disable NTP Server ===============================")
    else:
        # NTP未安装，跳过
        print("=============================== Check no exist NTP package ==================================")


def install_chrony():
    print("=============================== install Chrony package ===============================")
    returncode = subprocess.call(["yum", "install", "-y", "chrony"], shell=False)

    # 判断命令是否执行成功
    if returncode == 0:
        print("=============================== install Chrony Succssful ===============================")
    else:
        print("=============================== install Chrony Fail ===============================")


def is_line_valid(line):
    patterns = [
        r'\s*server.*iburst$',
        r'\s*server.*iburst prefer$',
        r'\s*allow.*0.0/16$',
        r'\s*local stratum'
    ]
    return any(re.match(pattern, line) for pattern in patterns)


def clear_conf():
    if os.path.exists('/etc/chrony.conf'):
        with open('/etc/chrony.conf', 'r') as f:
            lines = f.readlines()

        new_lines = []
        for line in lines:
            if not is_line_valid(line):
                new_lines.append(line)

        with open('/etc/chrony.conf', 'w') as f:
            f.writelines(new_lines)
    else:
        print("chrony.conf dosen't exists")


def master_chrony_conf(internet_ip):
    # 备份文件
    if not os.path.exists('/etc/chrony.conf.back'):
        print("chrony.backkkkkkkkk")
        os.rename("/etc/chrony.conf", "/etc/chrony.conf.back")

    # 创建配置文件
    with open('/etc/chrony.conf', 'a') as f:
        # 如果互联网源IP地址存在，则在配置文件中添加该IP
        if internet_ip:
            config_line = "server {} iburst\n".format(internet_ip)
            f.write(config_line)
        ip = subprocess.check_output(["ip", "route", "get", "192.168.1.1"],
            shell=False).decode('utf-8').split('src')[1].split()[0]
        network_segment = '%s.%s.0.0/16' % (ip.split(".")[0], ip.split(".")[1])
        f.write("allow {}\n".format(network_segment))
        f.write("local stratum 10")

    subprocess.call(["systemctl", "restart", "chronyd.service"], shell=False)
    subprocess.call(["systemctl", "enable", "chronyd.service"], shell=False)
    output = subprocess.Popen(["systemctl", "status", "chronyd.service"], shell=False,
        stdout=subprocess.PIPE).stdout.read().decode("utf-8")
    match = re.search(r'Active:\s+(\w+\s+\(\w+\))', output)
    if match:
        status = match.group(1)
        if status == "active (running)":
            print("\033[92m=============================== Successful ===============================\033[0m")
            print(
                "\033[92mThe configuration is complete. Wait 1-6 minutes and run 'chronyc sources' command to query the"
                "configuration result\033[0m")
        else:
            print("\033[91mCheck the Chrony status\033[0m")


def main(internet_ip):
    check_ntp()
    install_chrony()
    clear_conf()
    master_chrony_conf(internet_ip)


if __name__ == "__main__":
    validate_script_params()
    if ARGV_INPUT_DIC.get("internet_domain_name") == 'zzzz' and ARGV_INPUT_DIC.get("internet_ip") == 'zzzz':
        main(internet_ip=None)
    elif ARGV_INPUT_DIC.get("internet_domain_name") == 'zzzz' and ARGV_INPUT_DIC.get("internet_ip") != 'zzzz':
        internet_ip = ARGV_INPUT_DIC.get("internet_ip")
        main(internet_ip)
    elif ARGV_INPUT_DIC.get("internet_domain_name") != 'zzzz' and ARGV_INPUT_DIC.get("internet_ip") == 'zzzz':
        internet_domain_name = ARGV_INPUT_DIC.get("internet_domain_name")
        main(internet_domain_name)
    else:
        internet_domain_name = ARGV_INPUT_DIC.get("internet_domain_name")
        main(internet_domain_name)
