#!/usr/bin/env python
# -*- coding: utf-8 -*-


import os
import re
import subprocess
import sys

REMOTE_PARAMETERS = " ".join(sys.argv[1:])

ARGV_EXAMPLE_DIC = {"ntp_master_ip": "xxxx", "ntp_slaver_ip": "xxxx"}
# 存放输入的参数值
ARGV_INPUT_DIC = ARGV_EXAMPLE_DIC


def script_params_example():
    message = "parameter example: "
    for key, value in ARGV_EXAMPLE_DIC.items():
        message = message + key + "=" + value + " "
    return message.strip()


def validate_script_params():
    if len(REMOTE_PARAMETERS) == 0:
        print('Please enter parameters, ' + script_params_example())
        exit(255)
    tmp_argv = REMOTE_PARAMETERS.split(" ")
    for item in tmp_argv:
        argv = item.split("=")
        if len(argv) != 2:
            print('Invalid parameters format, ' + script_params_example())
            exit(255)
        if argv[0] not in ARGV_EXAMPLE_DIC.keys():
            print('Parameter [ ' + argv[0] + ' ] is not defined, ' + script_params_example())
            exit(255)
        if argv[1].strip() == "" and ARGV_EXAMPLE_DIC.get(argv[0]) == "xxxx":
            print('Parameter [ ' + argv[0] + ' ] cannot be empty.')
            exit(255)
        ARGV_INPUT_DIC[argv[0]] = argv[1]

    for key, value in ARGV_INPUT_DIC.items():
        if value == "xxxx":
            print('Parameter [ ' + key + ' ] cannot be empty.')
            exit(255)

    if len(ARGV_EXAMPLE_DIC) != len(ARGV_INPUT_DIC):
        print('Number of parameters does not match, ' + script_params_example())
        exit(255)
    validate_business_params()


def validate_business_params():
    ntp_master_ip = ARGV_INPUT_DIC.get("ntp_master_ip")
    ntp_slaver_ip = ARGV_INPUT_DIC.get("ntp_slaver_ip")
    local_ip = subprocess.Popen(["hostname", "-i"], shell=False, stdout=subprocess.PIPE).stdout.read().strip()
    if local_ip == ntp_master_ip or local_ip == ntp_slaver_ip:
        print("The client cannot be deployed on the master/slaver node.")
        exit(255)

    if ntp_master_ip == ntp_slaver_ip:
        print("The master/slaver IP addresses are the same. Please check the parameters.")
        exit(255)

    pattern = r'^(?:(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.){3}(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)$'
    if not bool(re.match(pattern, ntp_master_ip)) or not bool(re.match(pattern, ntp_slaver_ip)):
        print("\033[1;31mError:\033[0m Invalid IPv4 address")
        exit(255)

    print("Checking IP connectivity... Please wait.")
    try:
        output = subprocess.check_output(["ping", "-c", "1", ntp_master_ip], shell=False,
            stderr=subprocess.STDOUT, universal_newlines=True, timeout=2)
    except (subprocess.CalledProcessError, subprocess.TimeoutExpired):
        print("\033[1;31mError:\033[0m The ntp_master_ip cannot be connected.")
        exit(255)

    try:
        output = subprocess.check_output(["ping", "-c", "1", ntp_slaver_ip], shell=False,
            stderr=subprocess.STDOUT, universal_newlines=True, timeout=2)
    except (subprocess.CalledProcessError, subprocess.TimeoutExpired):
        print("\033[1;31mError:\033[0m The ntp_slaver_ip cannot be connected.")
        exit(255)


def check_chrony():
    # 检查是否安装Chrony包
    process1 = subprocess.Popen(["rpm", "-qa"], shell=False, stdout=subprocess.PIPE)
    process2 = subprocess.Popen(["grep", "chrony"], shell=False, stdin=process1.stdout, stdout=subprocess.PIPE)
    stdout, stderr = process2.communicate(timeout=5)
    returncode = process2.returncode

    if returncode == 0:
        # 关闭服务并禁止开机启动
        command = ["systemctl", "stop", "chronyd.service"]
        subprocess.Popen(command, shell=False, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        command = ["systemctl", "disable", "chronyd.service"]
        subprocess.Popen(command, shell=False, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        print("=============================== stop && disable Chrony Server   ===============================")
    else:
        # Chrony未安装，跳过
        print("=============================== check no exist Chrony package   ===============================")


def install_ntp():
    print("=============================== install NTP package   ===============================")
    returncode = subprocess.call(["yum", "install", "-y", "ntp"], shell=False)

    # 判断命令是否执行成功
    if returncode == 0:
        print("=============================== install NTP Succssful ===============================")
    else:
        print("=============================== install NTP Fail ===============================")
        exit(255)


def is_line_valid(line):
    patterns = [
        r'\s*server.*iburst$',
        r'\s*server.*iburst prefer$',
        r'\s*fudge 127\.127\.1\.0 stratum'
    ]
    return any(re.match(pattern, line) for pattern in patterns)


def clear_conf():
    if os.path.exists('/etc/ntp.conf'):
        with open('/etc/ntp.conf', 'r') as f:
            lines = f.readlines()

        new_lines = []
        for line in lines:
            if not is_line_valid(line) and 'restrict default kod nomodify notrap nopeer noquery' not in line:
                new_lines.append(line)

        with open('/etc/ntp.conf', 'w') as f:
            f.writelines(new_lines)
    else:
        print("ntp.conf dosen't exists")
        exit(255)


def client_ntp_conf(ntp_master_ip, ntp_slaver_ip):
    # 判断文件是否存在
    if not os.path.exists('/etc/ntp.conf.back'):
        # 如果文件不存在，执行rename操作
        os.rename("/etc/ntp.conf", "/etc/ntp.conf.back")

    # 创建配置文件
    with open('/etc/ntp.conf', 'a') as f:
        f.write("server {} iburst prefer\n".format(ntp_master_ip))
        f.write("server {} iburst".format(ntp_slaver_ip))

    subprocess.call(["systemctl", "enable", "ntpd.service"], shell=False)
    subprocess.call(["systemctl", "restart", "ntpd.service"], shell=False)
    output = subprocess.Popen(["systemctl", "status", "ntpd.service"], shell=False,
        stdout=subprocess.PIPE).stdout.read()
    output = output.decode()
    match = re.search(r'Active:\s+(\w+\s+\(\w+\))', output)
    if match:
        status = match.group(1)
        if status == "active (running)":
            print("\033[92m=============================== Successful ===============================\033[0m")
            print(
                "\033[92mThe configuration is complete. Wait 5-10 minutes and run 'ntpq -p' command on the client to "
                "query the configuration result\033[0m")
        else:
            print("\033[91mCheck the NTP status\033[0m")
            exit(255)


def main(ntp_master_ip, ntp_slaver_ip):
    check_chrony()
    install_ntp()
    clear_conf()
    client_ntp_conf(ntp_master_ip, ntp_slaver_ip)


if __name__ == "__main__":
    validate_script_params()
    ntp_master_ip = ARGV_INPUT_DIC.get("ntp_master_ip")
    ntp_slaver_ip = ARGV_INPUT_DIC.get("ntp_slaver_ip")
    main(ntp_master_ip, ntp_slaver_ip)
