#!/usr/bin/env python
# -*- coding: utf-8 -*-


import os
import re
import subprocess
import sys

# 需要传的参数:
# internet_ip=互联网时间源ip
# internet_domain_name=互联网时间源域名
# 可只传其中之一，都传只取internet_domain_name
REMOTE_PARAMETERS = " ".join(sys.argv[1:])

ARGV_EXAMPLE_DIC = {"internet_ip": "zzzz", "internet_domain_name": "zzzz"}
# 存放输入的参数值
ARGV_INPUT_DIC = ARGV_EXAMPLE_DIC


def script_params_example():
    message = "parameter example: "
    for key, value in ARGV_EXAMPLE_DIC.items():
        message = message + key + "=" + value + " "
    return message.strip()


def validate_script_params():
    if len(REMOTE_PARAMETERS) == 0:
        validate_business_params()

    else:
        tmp_argv = REMOTE_PARAMETERS.split(" ")
        for item in tmp_argv:
            argv = item.split("=")
            if len(argv) != 2:
                print('Invalid parameters format, ' + script_params_example())
                exit(255)
            if argv[0] not in ARGV_EXAMPLE_DIC.keys():
                print('Parameter [ ' + argv[0] + ' ] is not defined, ' + script_params_example())
                exit(255)
            if argv[1].strip() == "" and ARGV_EXAMPLE_DIC.get(argv[0]) == "xxxx":
                print('Parameter [ ' + argv[0] + ' ] cannot be empty.')
                exit(255)
            ARGV_INPUT_DIC[argv[0]] = argv[1]

        for key, value in ARGV_INPUT_DIC.items():
            if value == "xxxx":
                print('Parameter [ ' + key + ' ] cannot be empty.')
                exit(255)

        if len(ARGV_EXAMPLE_DIC) != len(ARGV_INPUT_DIC):
            print('Number of parameters does not match, ' + script_params_example())
            exit(255)
        validate_business_params()


def validate_business_params():
    if ARGV_INPUT_DIC.get("internet_domain_name") != 'zzzz':
        internet_domain_name = ARGV_INPUT_DIC.get("internet_domain_name")
        pattern = r'^(?:[a-z0-9](?:[a-z0-9-]*[a-z0-9])?\.)+[a-z0-9](?:[a-z0-9-]*[a-z0-9])?$'
        if not bool(re.match(pattern, internet_domain_name)):
            print("\033[1;31mError:\033[0m Invalid Domain Name")
            exit(255)

        print("Checking Domain Name connectivity... Please wait.")
        try:
            output = subprocess.check_output(["ping", "-c", "1", internet_domain_name], shell=False,
                stderr=subprocess.STDOUT, universal_newlines=True, timeout=2)
        except (subprocess.CalledProcessError, subprocess.TimeoutExpired):
            print("\033[1;31mError:\033[0m The Domain Name cannot be connected.")
            exit(255)

    if ARGV_INPUT_DIC.get("internet_ip") != 'zzzz':
        internet_ip = ARGV_INPUT_DIC.get("internet_ip")
        pattern = r'^(?:(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.){3}(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)$'
        if not bool(re.match(pattern, internet_ip)):
            print("\033[1;31mError:\033[0m Invalid IPv4 address")
            exit(255)

        print("Checking IP connectivity... Please wait.")
        try:
            output = subprocess.check_output(["ping", "-c", "1", internet_ip], shell=False,
                stderr=subprocess.STDOUT, universal_newlines=True, timeout=2)
        except (subprocess.CalledProcessError, subprocess.TimeoutExpired):
            print("\033[1;31mError:\033[0m The internet_ip cannot be connected.")
            exit(255)


def check_chrony():
    # 检查是否安装Chrony包
    process1 = subprocess.Popen(["rpm", "-qa"], shell=False, stdout=subprocess.PIPE)
    process2 = subprocess.Popen(["grep", "chrony"], shell=False, stdin=process1.stdout, stdout=subprocess.PIPE)
    stdout, stderr = process2.communicate(timeout=5)
    returncode = process2.returncode

    if returncode == 0:
        # 关闭服务并禁止开机启动
        command = ["systemctl", "stop", "chronyd.service"]
        subprocess.Popen(command, shell=False, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        command = ["systemctl", "disable", "chronyd.service"]
        subprocess.Popen(command, shell=False, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        print("=============================== Stop && Disable Chrony Server   ===============================")
    else:
        # Chrony未安装，跳过
        print("=============================== Check no exist Chrony package   ==================================")


def install_ntp():
    print("=============================== install NTP package   ===============================")
    returncode = subprocess.call(["yum", "install", "-y", "ntp"], shell=False)

    # 判断命令是否执行成功
    if returncode == 0:
        print("=============================== install NTP Succssful ===============================")
    else:
        print("=============================== install NTP Fail ===============================")
        exit(255)


def is_line_valid(line):
    patterns = [
        r'\s*server.*iburst$',
        r'\s*server.*iburst prefer$',
        r'\s*fudge 127\.127\.1\.0 stratum'
    ]
    return any(re.match(pattern, line) for pattern in patterns)


def clear_conf():
    if os.path.exists('/etc/ntp.conf'):
        with open('/etc/ntp.conf', 'r') as f:
            lines = f.readlines()

        new_lines = []
        for line in lines:
            if not is_line_valid(line) and 'restrict default kod nomodify notrap nopeer noquery' not in line:
                new_lines.append(line)

        with open('/etc/ntp.conf', 'w') as f:
            f.writelines(new_lines)
    else:
        print("ntp.conf dosen't exists")
        exit(255)


def master_ntp_conf(internet_ip):
    config = """server 127.127.1.0 iburst
fudge 127.127.1.0 stratum 10
restrict default kod nomodify notrap nopeer noquery
"""

    # 判断文件是否存在
    if not os.path.exists('/etc/ntp.conf.back'):
        # 如果文件不存在，执行rename操作
        os.rename("/etc/ntp.conf", "/etc/ntp.conf.back")

    # 创建配置文件
    with open('/etc/ntp.conf', 'a') as f:
        # 如果互联网源IP地址存在，则在配置文件中添加该IP
        if internet_ip:
            config_line = "server {} iburst\n".format(internet_ip)
            f.write(config_line)
        f.write(config)

    subprocess.call(["systemctl", "enable", "ntpd.service"], shell=False)
    subprocess.call(["systemctl", "restart", "ntpd.service"], shell=False)
    output = subprocess.Popen(["systemctl", "status", "ntpd.service"], shell=False,
        stdout=subprocess.PIPE).stdout.read()
    output = output.decode()
    match = re.search(r'Active:\s+(\w+\s+\(\w+\))', output)
    if match:
        status = match.group(1)
        if status == "active (running)":
            print("\033[92m=============================== Successful ===============================\033[0m")
            print(
                "\033[92mThe configuration is complete. Wait 5-10 minutes and run 'ntpq -p' command on the client to "
                "query the configuration result\033[0m")
        else:
            print("\033[91mCheck the NTP status\033[0m")
            exit(255)


def main(internet_ip):
    check_chrony()
    install_ntp()
    clear_conf()
    master_ntp_conf(internet_ip)


if __name__ == "__main__":
    validate_script_params()
    if ARGV_INPUT_DIC.get("internet_domain_name") == 'zzzz' and ARGV_INPUT_DIC.get("internet_ip") == 'zzzz':
        main(internet_ip=None)
    elif ARGV_INPUT_DIC.get("internet_domain_name") == 'zzzz' and ARGV_INPUT_DIC.get("internet_ip") != 'zzzz':
        internet_ip = ARGV_INPUT_DIC.get("internet_ip")
        main(internet_ip)
    elif ARGV_INPUT_DIC.get("internet_domain_name") != 'zzzz' and ARGV_INPUT_DIC.get("internet_ip") == 'zzzz':
        internet_domain_name = ARGV_INPUT_DIC.get("internet_domain_name")
        main(internet_domain_name)
    else:
        internet_domain_name = ARGV_INPUT_DIC.get("internet_domain_name")
        main(internet_domain_name)
